import datetime

import pytz
from django.conf import settings
from django.utils.safestring import EscapeData, SafeData
from jinja2 import environmentfilter, Markup, Undefined

if settings.TIME_ZONE:
    local_tzinfo = pytz.timezone(settings.TIME_ZONE)
else:
    local_tzinfo = pytz.utc


def localtime(value):
    return value.astimezone(local_tzinfo)


def is_naive(value):
    """
    Determines if a given datetime.datetime is naive.

    The logic is described in Python's docs:
    http://docs.python.org/library/datetime.html#datetime.tzinfo
    """
    return value.tzinfo is None or value.tzinfo.utcoffset(value) is None


def template_localtime(value, use_tz=None):
    """
    Checks if value is a datetime and converts it to local time if necessary.

    If use_tz is provided and is not None, that will force the value to
    be converted (or not), overriding the value of settings.USE_TZ.

    This function is designed for use by the template engine.
    """
    should_convert = (isinstance(value, datetime.datetime)
        and (settings.USE_TZ if use_tz is None else use_tz)
        and not is_naive(value)
        and getattr(value, 'convert_to_local_time', True))
    return localtime(value) if should_convert else value


def django_filter_to_jinja2(filter_func):
    """
    Note: Due to the way this function is used by
    ``coffin.template.Library``, it needs to be able to handle native
    Jinja2 filters and pass them through unmodified. This necessity
    stems from the fact that it is not always possible to determine
    the type of a filter.

    TODO: Django's "func.is_safe" is not yet handled
    """
    def _convert_out(v):
        if isinstance(v, SafeData):
            return Markup(v)
        if isinstance(v, EscapeData):
            return Markup.escape(v)  # not 100% equivalent, see mod docs
        return v

    def _convert_in(v):
        if isinstance(v, Undefined):
            # Essentially the TEMPLATE_STRING_IF_INVALID default
            # setting. If a non-default is set, Django wouldn't apply
            # filters. This is something that we neither can nor want to
            # simulate in Jinja.
            return ''
        return v

    def conversion_wrapper(value, *args, **kwargs):
        result = filter_func(_convert_in(value), *args, **kwargs)
        return _convert_out(result)
    # Jinja2 supports a similar machanism to Django's
    # ``needs_autoescape`` filters: environment filters. We can
    # thus support Django filters that use it in Jinja2 with just
    # a little bit of argument rewriting.
    if hasattr(filter_func, 'needs_autoescape'):
        @environmentfilter
        def autoescape_wrapper(environment, *args, **kwargs):
            kwargs['autoescape'] = environment.autoescape
            return conversion_wrapper(*args, **kwargs)
        return autoescape_wrapper
    else:
        return conversion_wrapper



from jinja2 import Environment
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase


class TestEnvironment(TestCase):

    def test_getattr(self):
        from django_cofingo import env

        class Foo(object):
            @property
            def item(self):
                raise ObjectDoesNotExist()

        tmpl = env.from_string('{{ foo.item }}', {'foo': Foo()})
        self.assertEqual(tmpl.render(), '')

    def test_getattr(self):
        from django_cofingo import env

        class Foo(object):
            def __getitem__(self, key):
                raise ObjectDoesNotExist()

        tmpl = env.from_string("{{ foo['item'] }}", {'foo': Foo()})
        self.assertEqual(tmpl.render(), '')



from django.conf.urls.defaults import *

urlpatterns = patterns('',
    (r'^url_test/', include('apps.urls_app.urls')),

    (r'^fullstack/', include('apps.fullstack_app.urls')),

    # These two are used to test that our url-tag implementation can
    # deal with application namespaces / the "current app".
    (r'^app/one/', include('apps.urls_app.urls',
                           app_name="testapp", namespace="testapp")),

    (r'^app/two/', include('apps.urls_app.urls',
                           app_name="testapp", namespace="two")),
)



import os


def load_tests(loader, standard_tests, pattern):
    """This function is automatically called by unittest2 to discover tests
    within this module.

    """
    package_tests = loader.discover(
        start_dir=os.path.dirname(__file__),
        pattern='test*.py',
        top_level_dir=os.path.join(os.path.dirname(__file__), '..'))
    standard_tests.addTests(package_tests)
    return standard_tests

# Make sure that nose doesn't collect load_test as a testcase
load_tests.__test__ = False




from django.test import TestCase

from django_cofingo.library import Library


class TestLibrary(TestCase):
    def test_add_filter(self):
        library = Library()

        def func(value):
            return 'filter({})'.format(value)

        library.filter(func)



import os
import sys

from setuptools import setup, find_packages, Command

install_requires = [
    'Django>=1.3',
    'Jinja2>=2.5',
    'pytz',
]


class RunTests(Command):
    """From django-celery"""
    description = "Run the tests"
    user_options = []
    extra_env = {}
    extra_args = ['django_cofingo']

    def run(self):
        for env_name, env_value in self.extra_env.items():
            os.environ[env_name] = str(env_value)

        sys.path.append(os.path.join(
            os.path.dirname(__file__), 'django_cofingo', 'tests'))

        from django.conf import settings
        from django.core.management import execute_manager

        if not settings.configured:
            settings.configure(
                DATABASE_ENGINE='sqlite3',
                DATABASES={
                    'default': {
                        'ENGINE': 'django.db.backends.sqlite3',
                        'TEST_NAME': 'cofingo_tests.db',
                    },
                },
                INSTALLED_APPS=[
                    'django.contrib.auth',
                    'django.contrib.contenttypes',
                    'django.contrib.admin',
                    'django.contrib.sessions',
                    'django.contrib.sites',

                    'django_cofingo',
                    'apps.urls_app',
                    'apps.fullstack_app',
                ],
                TEMPLATE_LOADERS=[
                    'django_cofingo.Loader'
                ],
                ROOT_URLCONF='apps.urls',
                DEBUG=False,
                SITE_ID=1,
                TEMPLATE_DEBUG=True,
                TEMPLATE_DIRS=[],
                SETTINGS_MODULE='apps'
            )

        prev_argv = list(sys.argv)
        try:
            sys.argv = [__file__, "test"] + self.extra_args
            execute_manager(settings, argv=sys.argv)
        finally:
            sys.argv = prev_argv

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass


readme = []
with open('README.rst', 'r') as fh:
    readme = fh.readlines()


setup(
    name='django-cofingo',
    version='0.2.2',
    license='BSD',
    url='http://github.com/mvantellingen/django-cofingo',
    author='Michael van Tellingen',
    author_email='michaelvantellingen@gmail.com',
    description='Jinja2 template renderer for Django',
    long_description=''.join(readme),
    install_requires=install_requires,
    zip_safe=False,
    cmdclass={"test": RunTests},
    packages=find_packages(exclude=('tests')),
    include_package_data=True,
)



from django.conf.urls.defaults import *

urlpatterns = patterns('apps.urls_app',
    # Test urls for testing reverse lookups
    url(r'^$', 'views.index', name='the-index-view'),
    (r'^sum/(?P<left>\d+),(?P<right>\d+)$', 'views.sum'),
)
